@@ -1,7 +1,7 @@
 // C++ source
 // This file is part of RGL.
 //
-// $Id: rglview.cpp 1115 2014-07-18 13:51:22Z murdoch $
+// $Id: rglview.cpp 1138 2014-09-25 17:52:54Z murdoch $
 
 
 
@@ -163,8 +163,8 @@ void RGLView::buttonPress(int button, int mouseX, int mouseY)
 	  subscene->translateCoords(&mouseX, &mouseY);
 	  drag = button;
 	  activeSubscene = subscene->getObjID();
-	  vwidth = subscene->pviewport[2];
-	  vheight = subscene->pviewport[3];
+	  vwidth = subscene->pviewport.width;
+	  vheight = subscene->pviewport.height;
 	  windowImpl->captureMouse(this);	  
 	  (this->*ButtonBeginFunc[button-1])(mouseX, mouseY);
 	}
@@ -193,8 +193,8 @@ void RGLView::mouseMove(int mouseX, int mouseY)
     	}
     	subscene->translateCoords(&mouseX, &mouseY);
 
-	vwidth = subscene->pviewport[2]; /* These may have changed */
-	vheight = subscene->pviewport[3];    	  
+	vwidth = subscene->pviewport.width; /* These may have changed */
+	vheight = subscene->pviewport.height;    	  
     	
   	mouseX = clamp(mouseX, 0, vwidth-1);
   	mouseY = clamp(mouseY, 0, vheight-1);
@@ -981,3 +981,7 @@ bool RGLView::postscript(int formatID, const char* filename, bool drawText)
   return success;
 }
 
+void RGLView::setNULLActive(bool nullActive)
+{
+  renderContext.NULLActive = nullActive;
+}
