@@ -1,4 +1,7 @@
 library(rgl)
 
+# regression :  this failed for headless tests
+par3d(userMatrix = diag(4))
+
 for(demo in demo(package="rgl")$results[,"Item"]) 
   demo(demo, package="rgl", character.only=TRUE)
